import 'dart:async';
import 'dart:developer';
import 'dart:io';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

void main() {
  WidgetsFlutterBinding.ensureInitialized();
  runZonedGuarded(() {
    runApp(const MyApp());
  }, (error, stackTrace) {
    log('runZonedGuarded:  $error');
  });
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      debugShowCheckedModeBanner: false,
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(),
    );
  }
}

class MyHomePage extends StatefulWidget {
  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  TextEditingController ipC = TextEditingController(text: "192.168.1.248");
  TextEditingController portC = TextEditingController(text: "35000");

  bool loading = false;
  bool connecting = false;

  Future<bool> tryConnectSocket() async {
    String ip = "192.168.9.101";
    int port = 35000;
    try {
      RawSocketOption option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0xffff, /* SO_KEEPALIVE */ 0x0008, true);
      if (Platform.isAndroid) {
        option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0x1, /* SO_KEEPALIVE */ 0x0009, true);
      } else {
        option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0xffff, /* SO_KEEPALIVE */ 0x0008, true);
      }

      Socket s = await Socket.connect(ip, 35000, timeout: const Duration(seconds: 10)).timeout(const Duration(seconds: 10),
          onTimeout: () {
        log("ERROR ON SOCKET TIMEOUT");
        return Socket.connect(ip, port);
      }).catchError((e) {
        log("ERROR ON SOCKET $e");
        return Socket.connect(ip, port);
      }).onError((error, stackTrace) {
        log("ERROR ON SOCKET $error");
        return Socket.connect(ip, port);
      }).then((value) {
        value.setRawOption(option);
        return value;
      });

      log("DONE");
      s.listen((msg) {}, cancelOnError: false, onError: (e) {
        log("ERROR ON SOCKET $e");
      }, onDone: () {
        log("Socket DONE");
      });
      log("DONE");
      return true;
    } catch (e) {
      log("ERROR ON LISTENING TO SOCKET $e");
      return false;
    }
  }

  @override
  Widget build(BuildContext context) {
    ThemeData theme = Theme.of(context);

    return Scaffold(
      appBar: AppBar(
        title: const Text("APP"),
      ),
      body: Column(
        children: [
          const Divider(),
          Container(
            padding: const EdgeInsets.symmetric(horizontal: 24),
            width: MediaQuery.of(context).size.width,
            color: Colors.white,
            child: Column(
              children: [
                Row(
                  children: [
                    Expanded(
                        flex: 4,
                        child: CupertinoTextField(
                          keyboardType: const TextInputType.numberWithOptions(signed: true, decimal: true),
                          controller: ipC,
                          placeholder: "ip e.g. 192.168.1.7",
                        )),
                    const SizedBox(width: 12),
                    Expanded(
                        flex: 2,
                        child: CupertinoTextField(
                          keyboardType: const TextInputType.numberWithOptions(signed: true),
                          controller: portC,
                          placeholder: "port e.g. 7535",
                        )),
                    const SizedBox(width: 12),
                    TextButton(
                        onPressed: () async {
                          tryConnectSocket().then((value) {
                            print(value);
                          });
                        },
                        child: connecting ? const Text("...") : const Text("Connect")),
                  ],
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
