import 'dart:developer';

import '../../classes/device.dart';
import 'package:xml/xml.dart';
import '../../constants.dart';
import '../classes.dart';

class BcDevice extends Device {
  BcDevice(
      {required BcDeviceParameter deviceParameter,
      required String deviceIndex,
      required String deviceName,
      required String deviceParameterType,
      required List<InterfaceMode> supportedInterfaceModes,
      required List<Device> devices})
      : super(
            deviceParameter: deviceParameter,
            devices: devices,
            deviceParameterType: deviceParameterType,
            deviceName: deviceName,
            deviceIndex: deviceIndex,
            supportedInterfaceModes: supportedInterfaceModes);

  factory BcDevice.fromDevice(Device d) {
    return BcDevice(
      deviceParameter: BcDeviceParameter.fromXML(d.deviceParameter.toXML()),
      deviceIndex: d.deviceIndex,
      deviceName: d.deviceName,
      deviceParameterType: d.deviceParameterType,
      supportedInterfaceModes: d.supportedInterfaceModes,
      devices: d.devices,
    );
  }

  Future<CuppsCommandResponse> lockDevice() async {
    CuppsCommandResponse lockRes = await lock();
    log("$lockRes");
    return lockRes;
    // print(lockRes);
  }

  Future<CuppsCommandResponse> unlockDevice() async {
    CuppsCommandResponse unlockRes = await unlock();
    log("$unlockRes");
    return unlockRes;
    // print(lockRes);
  }
}

class BcDeviceParameter extends DeviceParameter {
  final String numberVisualIndicator;

  BcDeviceParameter({
    required VendorModelInfo vendorModelInfo,
    required IpAndPort ipAndPort,
    required Status status,
    required this.numberVisualIndicator,
  }) : super(
            vendorModelInfo: vendorModelInfo,
            ipAndPort: ipAndPort,
            status: status);

  factory BcDeviceParameter.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? vendorModelInfoElement =
        document.firstChild!.getElement("vendorModelInfo");
    XmlElement? ipAndPortElement = document.firstChild!.getElement("ipAndPort");
    XmlElement? statusElement = document.firstChild!.getElement("bcStatus");

    VendorModelInfo vendorModelInfo =
        VendorModelInfo.fromXML(vendorModelInfoElement!.toXmlString());
    IpAndPort ipAndPort = IpAndPort.fromXML(ipAndPortElement!.toXmlString());
    BcStatus status = BcStatus.fromXML(statusElement!.toXmlString());

    String numberVisualIndicator = document.firstChild!.attributes
        .firstWhere(
            (element) => element.name.toString() == "numberVisualIndicator",
            orElse: orElseEmpty)
        .value;
    return BcDeviceParameter(
        vendorModelInfo: vendorModelInfo,
        ipAndPort: ipAndPort,
        status: status,
        numberVisualIndicator: numberVisualIndicator);
  }

  @override
  String toXML() {
    return '''<bcDeviceParameter numberVisualIndicator="$numberVisualIndicator" >${vendorModelInfo.toXML()}${ipAndPort.toXML()}${status.toXML()}</bcDeviceParameter>''';
  }
}

class BcStatus extends Status {
  BcStatus({
    required String desc,
    required String init,
    required String powerOff,
    required String ready,
    required String unknown,
  }) : super(
            desc: desc,
            init: init,
            powerOff: powerOff,
            ready: ready,
            unknown: unknown);

  factory BcStatus.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String desc = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "desc",
            orElse: orElseEmpty)
        .value;
    String init = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "init")
        .value;
    String powerOff = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "powerOff")
        .value;
    String ready = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "ready")
        .value;
    String unknown = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "unknown")
        .value;
    return BcStatus(
        desc: desc,
        init: init,
        powerOff: powerOff,
        ready: ready,
        unknown: unknown);
  }

  @override
  String get image {
    if (isOff) return AssetImages.bcIconsAddressDisconnect;
    if (isLocked) return AssetImages.bcIconsAddressLocked;
    if (isInit) return AssetImages.bcIconsAddressIn;
    if (isReady) return AssetImages.bcIconsAddressActive;
    return AssetImages.bcIconsAddressNatural;
  }

  @override
  String toXML() {
    return '''<bcStatus desc="$desc" init="$init" powerOff="$powerOff" ready="$ready" unknown="$unknown"  />''';
  }
}
