import '../../classes/Responses/aea_response_res.dart';
import '../../classes/Requests/aea_request_req.dart';
import '../../classes/device.dart';
import '../../constants.dart';
import 'package:xml/xml.dart';
import '../classes.dart';

class BgDevice extends Device {
  BgDevice(
      {required BgDeviceParameter deviceParameter,
      required String deviceIndex,
      required String deviceName,
      required String deviceParameterType,
      required List<InterfaceMode> supportedInterfaceModes,
      required List<Device> devices})
      : super(
            deviceParameter: deviceParameter,
            devices: devices,
            deviceParameterType: deviceParameterType,
            deviceName: deviceName,
            deviceIndex: deviceIndex,
            supportedInterfaceModes: supportedInterfaceModes);

  factory BgDevice.fromDevice(Device d) {
    return BgDevice(
      deviceParameter: BgDeviceParameter.fromXML(d.deviceParameter.toXML()),
      deviceIndex: d.deviceIndex,
      deviceName: d.deviceName,
      deviceParameterType: d.deviceParameterType,
      supportedInterfaceModes: d.supportedInterfaceModes,
      devices: d.devices,
    );
  }

  Future<CuppsCommandResponse> auth({required String airlineID, String mode  = "HDC",String font = "L",}) async {

    // AeaRequestRequest req = AeaRequestRequest(messageID: currentMessageID, texts: ["EP#AIRLINEID=$airlineID#HARDCODE=HDC#FONT=L"],binaries: []);
    final setLogoRes = await aeaRequest("EP#AIRLINEID=$airlineID#HARDCODE=HDC#FONT=$font#ERR3IGN=Y");
    // final setLogoRes = await aeaRequest("EP#AIRLINEID=$airlineID#HARDCODE=$mode#FONT=$font");
    return setLogoRes;
    // print(setLogoRes);
    // send(xml: req.toXML());
  }

  sendCommand(String text) {
    AeaRequestRequest req = AeaRequestRequest(
        messageID: currentMessageID, texts: [text], binaries: []);
    send(xml: req.toXML());
  }

  displayText(String text) async {
    AeaRequestRequest req = AeaRequestRequest(
        messageID: currentMessageID, texts: ["MG#B#A$text"], binaries: []);
    String displayResXML = await send(xml: req.toXML());
    AeaResponseResponse displayRes = AeaResponseResponse.fromXML(displayResXML);
    if (displayRes.result.toLowerCase() == "ok") {
      return CuppsCommandResponse(
          msg: "Message Displayed Successfully", status: true);
    } else {
      return CuppsCommandResponse(msg: displayRes.result, status: false);
    }
  }
}

class BgDeviceParameter extends DeviceParameter {
  BgDeviceParameter({
    required VendorModelInfo vendorModelInfo,
    required IpAndPort ipAndPort,
    required Status status,
  }) : super(
            vendorModelInfo: vendorModelInfo,
            ipAndPort: ipAndPort,
            status: status);

  factory BgDeviceParameter.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? vendorModelInfoElement =
        document.firstChild!.getElement("vendorModelInfo");
    XmlElement? ipAndPortElement = document.firstChild!.getElement("ipAndPort");
    XmlElement? statusElement = document.firstChild!.getElement("bgStatus");

    VendorModelInfo vendorModelInfo =
        VendorModelInfo.fromXML(vendorModelInfoElement!.toXmlString());
    IpAndPort ipAndPort = IpAndPort.fromXML(ipAndPortElement!.toXmlString());
    BgStatus status = BgStatus.fromXML(statusElement!.toXmlString());

    ///todo handle bg poCapabilities in vendorinfo
    return BgDeviceParameter(
      vendorModelInfo: vendorModelInfo,
      ipAndPort: ipAndPort,
      status: status,
    );
  }

  @override
  String toXML() {
    return '''<bgDeviceParameter>
    ${vendorModelInfo.toXML()}
    ${ipAndPort.toXML()}
    ${status.toXML()}
    </bgDeviceParameter>
    ''';
  }
}

class BgStatus extends Status {
  final List<Status> status;

  BgStatus(
      {required String desc,
      required String init,
      required String powerOff,
      required String ready,
      required String unknown,
      required this.status})
      : super(
            desc: desc,
            init: init,
            powerOff: powerOff,
            ready: ready,
            unknown: unknown);

  factory BgStatus.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String desc = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "desc",
            orElse: orElseEmpty)
        .value;
    String init = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "init")
        .value;
    String powerOff = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "powerOff")
        .value;
    String ready = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "ready")
        .value;
    String unknown = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "unknown")
        .value;
    // List<Status> status = [
    //   BcStatus.fromXML(document.firstChild!.children[0].toString()),
    //   DdStatus.fromXML(document.firstChild!.children[1].toString()),
    //   MsStatus.fromXML(document.firstChild!.children[2].toString()),
    // ];
    return BgStatus(
        desc: desc,
        init: init,
        powerOff: powerOff,
        ready: ready,
        unknown: unknown,
        status: []);
  }

  @override
  String toXML() {
    return '''<bgStatus desc="$desc" init="$init" powerOff="$powerOff" ready="$ready" unknown="$unknown">
    ${status.map((e) => e.toXML()).join()}
    </bgStatus>
    ''';
  }

  @override
  String get image {
    if (isOff) return AssetImages.bgIconsAddressDisconnect;
    if (isInit) return AssetImages.bgIconsAddressIn;
    if (isReady) return AssetImages.bgIconsAddressActive;
    return AssetImages.bgIconsAddressNatural;
  }
}
