import '../../classes/Requests/print_request.dart';
import '../../classes/Responses/print_res.dart';
import '../../classes/device.dart';
import 'package:xml/xml.dart';
import '../../constants.dart';
import '../classes.dart';

class PrDevice extends Device {
  PrDevice(
      {required PrDeviceParameter deviceParameter,
      required String deviceIndex,
      required String deviceName,
      required String deviceParameterType,
      required List<InterfaceMode> supportedInterfaceModes,
      required List<Device> devices})
      : super(
            deviceParameter: deviceParameter,
            devices: devices,
            deviceParameterType: deviceParameterType,
            deviceName: deviceName,
            deviceIndex: deviceIndex,
            supportedInterfaceModes: supportedInterfaceModes);

  factory PrDevice.fromDevice(Device d) {
    return PrDevice(
      deviceParameter: PrDeviceParameter.fromXML(d.deviceParameter.toXML()),
      deviceIndex: d.deviceIndex,
      deviceName: d.deviceName,
      deviceParameterType: d.deviceParameterType,
      supportedInterfaceModes: d.supportedInterfaceModes,
      devices: d.devices,
    );
  }

  Future<CuppsCommandResponse> lockDevice() async {
    CuppsCommandResponse lockRes = await lock();
    return lockRes;
    // print(lockRes);
  }

  Future<CuppsCommandResponse> unlockDevice() async {
    CuppsCommandResponse lockRes = await unlock();
    return lockRes;
    // print(lockRes);
  }

  Future<CuppsCommandResponse> printPr(String text) async {
    PrintRequest req =
        PrintRequest(messageID: currentMessageID, printDocuments: [
      PrintDocument(
        textDocuments: [
          DocumentNode(texts: [TextNode(value: text)])
        ],
        pdfs: [],
        documentID: "$currentMessageID",
        stockName: 'letter',
      )
    ]);

    try {
      String printResXML = await send(xml: req.toXML());
      PrintResponse printRes = PrintResponse.fromXML(printResXML);
      if (printRes.result.toLowerCase() == "ok") {
        return CuppsCommandResponse(msg: printRes.result, status: true);
      } else {
        return CuppsCommandResponse(msg: printRes.result, status: false);
      }
    } catch (e) {
      return CuppsCommandResponse(msg:  "Device $deviceName send Exception: $e", status: false);
    }

    // print(printRes.toXML());
  }
}

class PrDeviceParameter extends DeviceParameter {
  final String supportsColor;
  final List<SupportedStock> supportedStocks;

  PrDeviceParameter(
      {required VendorModelInfo vendorModelInfo,
      required IpAndPort ipAndPort,
      required Status status,
      required this.supportsColor,
      required this.supportedStocks})
      : super(
            vendorModelInfo: vendorModelInfo,
            ipAndPort: ipAndPort,
            status: status);

  factory PrDeviceParameter.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? vendorModelInfoElement =
        document.firstChild!.getElement("vendorModelInfo");
    XmlElement? ipAndPortElement = document.firstChild!.getElement("ipAndPort");
    XmlElement? statusElement = document.firstChild!.getElement("prStatus");

    VendorModelInfo vendorModelInfo =
        VendorModelInfo.fromXML(vendorModelInfoElement!.toXmlString());
    IpAndPort ipAndPort = IpAndPort.fromXML(ipAndPortElement!.toXmlString());
    PrStatus status = PrStatus.fromXML(statusElement!.toXmlString());

    XmlElement? supportedStocksElement =
        document.firstChild!.getElement("supportedStocks");
    List<XmlElement> supporters = supportedStocksElement == null
        ? []
        : supportedStocksElement.findElements("supportedStock").toList();

    String supportsColor = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "supportsColor",
            orElse: orElseEmpty)
        .value;
    List<SupportedStock> supportedStocks =
        supporters.map((e) => SupportedStock.fromXML(e.toXmlString())).toList();
    return PrDeviceParameter(
        vendorModelInfo: vendorModelInfo,
        ipAndPort: ipAndPort,
        status: status,
        supportsColor: supportsColor,
        supportedStocks: supportedStocks);
  }

  @override
  String toXML() {
    return '''<prDeviceParameter supportsColor="$supportsColor" >
    ${vendorModelInfo.toXML()}
    ${ipAndPort.toXML()}
    ${status.toXML()}
    <supportedStocks>
    ${supportedStocks.map((e) => e.toXML()).join()}
    </supportedStocks>
    </prDeviceParameter>
    ''';
  }
}

class PrStatus extends Status {
  final String paperOut;
  final String paperJam;

  PrStatus({
    required String desc,
    required String init,
    required String powerOff,
    required String ready,
    required String unknown,
    required this.paperJam,
    required this.paperOut,
  }) : super(
            desc: desc,
            init: init,
            powerOff: powerOff,
            ready: ready,
            unknown: unknown);

  factory PrStatus.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String desc = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "desc",
            orElse: orElseEmpty)
        .value;
    String init = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "init")
        .value;
    String powerOff = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "powerOff")
        .value;
    String ready = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "ready")
        .value;
    String unknown = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "unknown")
        .value;
    String paperJam = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "paperJam")
        .value;
    String paperOut = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "paperOut")
        .value;
    return PrStatus(
        desc: desc,
        init: init,
        powerOff: powerOff,
        ready: ready,
        unknown: unknown,
        paperJam: paperJam,
        paperOut: paperOut);
  }

  @override
  String toXML() {
    return '''<prStatus desc="$desc" init="$init" powerOff="$powerOff" ready="$ready" unknown="$unknown" paperJam="$paperJam" paperOut="$paperOut"/>''';
  }

  @override
  String get image {
    if (isOff) return AssetImages.prIconsAddressDisconnect;
    if (isLocked) return AssetImages.prIconsAddressLocked;
    if (isInit) return AssetImages.prIconsAddressIn;
    if (isReady) return AssetImages.prIconsAddressActive;
    return AssetImages.prIconsAddressNatural;
  }
}
