import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class ApplicationInstBlockEvent extends Notify {
  final String user;
  final String workstation;
  final String maxInstances;
  final String applicationName;
  final String applicationVersion;

  ApplicationInstBlockEvent({
    required int messageID,
    required String messageName,
    required this.user,
    required this.workstation,
    required this.maxInstances,
    required this.applicationVersion,
    required this.applicationName,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "applicationInstBlockEvent");

  factory ApplicationInstBlockEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("applicationInstBlockEvent").first;
    String user = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "user", orElse: orElseEmpty).value;
    String applicationVersion = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationVersion", orElse: orElseEmpty).value;
    String applicationName = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationName", orElse: orElseEmpty).value;
    String maxInstances = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "maxInstances", orElse: orElseEmpty).value;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;

    return ApplicationInstBlockEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      user: user,
      workstation: workstation,
      maxInstances:maxInstances,
      applicationName: applicationName,
      applicationVersion: applicationVersion,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <applicationInstBlockEvent applicationName="$applicationName" applicationVersion="$applicationVersion" 
            maxInstances="$maxInstances" 
            user="$user" 
            workstation="$workstation" />
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
