import '../../classes/notify.dart';
import 'package:xml/xml.dart';
import '../device.dart';

class DeviceStatusNotification extends Notify {
  final Status status;

  DeviceStatusNotification({
    required int messageID,
    required String messageName,
    required this.status,
  }) : super(
            messageID: messageID,
            messageName: messageName,
            notificationType: NotificationType.unknown,
            name: "deviceStatusNotification");

  factory DeviceStatusNotification.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    // String xmlns = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns").value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    // String xsi = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns:xsi").value;
    // String result = document.rootElement.childElements.first.attributes.firstWhere((p0) => p0.name.toString() == "result",orElse: orElseEmpty).value;

    XmlElement resElement =
        document.rootElement.findAllElements("deviceStatusNotification").first;
    XmlElement statusElement = resElement.childElements
        .where((element) => element.name.toString().endsWith("Status"))
        .first;

    Status status = Status.fromXML(statusElement.toXmlString());
    return DeviceStatusNotification(
        messageName: messageName,
        messageID: int.parse(messageID),
        status: status);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <deviceStatusNotification>
                ${status.toXML()}
            </deviceStatusNotification>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
