import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class DLkdExitedEvent extends Notify {
  final String workstation;
  final String device;
  final String user;
  final String applicationName;
  final String applicationVersion;

  DLkdExitedEvent({
    required int messageID,
    required String messageName,
    required this.workstation,
    required this.device,
    required this.user,
    required this.applicationVersion,
    required this.applicationName,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "dLkdExitedEvent");

  factory DLkdExitedEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("dLkdExitedEvent").first;
    String workstation = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;
    String user = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "user", orElse: orElseEmpty).value;
    String applicationVersion = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationVersion", orElse: orElseEmpty).value;
    String applicationName = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationName", orElse: orElseEmpty).value;
    String device = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "device", orElse: orElseEmpty).value;

    return DLkdExitedEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      workstation: workstation,
      device: device,
      user: user,
      applicationName: applicationName,
      applicationVersion: applicationVersion,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <dLkdExitedEvent applicationName="$applicationName" applicationVersion="$applicationVersion" user="$user" 
            workstation="$workstation" device="$device" />
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
