import '../../classes/notify.dart';
import 'package:xml/xml.dart';
import '../classes.dart';
import '../device.dart';

class DStgExitedEvent extends Notify {
  final String device;

  DStgExitedEvent({
    required int messageID,
    required String messageName,
    required this.device,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "dStgExitedEvent");

  factory DStgExitedEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("dStgExitedEvent").first;
    String device = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "device", orElse: orElseEmpty).value;

    return DStgExitedEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      device: device,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
    <notify>
    <dStgExitedEvent device="$device"/>
    </notify>
    </cupps>
    ''';
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
