import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class PrintRequestStatusEvent extends Notify {
  final String documentID;
  final String status;

  PrintRequestStatusEvent({
    required int messageID,
    required String messageName,
    required this.documentID,
    required this.status,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "printRequestStatusEvent");

  factory PrintRequestStatusEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("printRequestStatusEvent").first;
    String documentID = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "documentID", orElse: orElseEmpty).value;
    String status = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "status", orElse: orElseEmpty).value;

    return PrintRequestStatusEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      documentID: documentID,
      status: status,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <printRequestStatusEvent documentID="$documentID" status="$status"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
