import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class PSpgEnteredEvent extends Notify {
  final String admin;

  PSpgEnteredEvent({
    required int messageID,
    required String messageName,
    required this.admin,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "pSpgEnteredEvent");

  factory PSpgEnteredEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("pSpgEnteredEvent").first;
    String admin = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "admin", orElse: orElseEmpty).value;

    return PSpgEnteredEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      admin: admin,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <pSpgEnteredEvent admin="$admin"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
