import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class UStgExitedEvent extends Notify {
  final String user;
  final String workstation;

  UStgExitedEvent({
    required int messageID,
    required String messageName,
    required this.user,
    required this.workstation,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "uStgExitedEvent");

  factory UStgExitedEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("uStgExitedEvent").first;
    String user = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "user", orElse: orElseEmpty).value;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;

    return UStgExitedEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      user: user,
      workstation: workstation,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <uStgExitedEvent user="$user" workstation="$workstation"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
