import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class WAltExitedEvent extends Notify {
  final String admin;
  final String message;
  final String workstation;

  WAltExitedEvent({
    required int messageID,
    required String messageName,
    required this.admin,
    required this.message,
    required this.workstation,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "wAltExitedEvent");

  factory WAltExitedEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("wAltExitedEvent").first;
    String admin = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "admin", orElse: orElseEmpty).value;
    String message = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "message", orElse: orElseEmpty).value;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;

    return WAltExitedEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      admin: admin,
      message: message,
      workstation: workstation,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <wAltExitedEvent admin="$admin" message="$message" workstation="$workstation"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
