import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class WStdEnteredEvent extends Notify {
  final String workstation;

  WStdEnteredEvent({
    required int messageID,
    required String messageName,
    required this.workstation,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "wStdEnteredEvent");

  factory WStdEnteredEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("wStdEnteredEvent").first;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;

    return WStdEnteredEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      workstation: workstation,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <wStdEnteredEvent workstation="$workstation"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
