import 'package:xml/xml.dart';

import '../classes.dart';

class AuthenticateRequest extends CuppsBase {
  final String airline;
  final String eventToken;
  final String platformDefinedParameter;
  final List<Application> applications;

  AuthenticateRequest({
    String? xmlns,
    String? xsi,
    required int messageID,
    required this.airline,
    required this.eventToken,
    required this.platformDefinedParameter,
    required this.applications,
  }) : super(messageID: messageID, messageName: "authenticateRequest");

  factory AuthenticateRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    String xmlns = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "xmlns")
        .value;
    String xsi = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "xmlns:xsi")
        .value;
    String airline = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "airline")
        .value;
    String eventToken = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "eventToken")
        .value;
    String platformDefinedParameter = document
        .rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "platformDefinedParameter")
        .value;
    List<Application> applicationsList = document
        .rootElement.firstChild!.firstChild!.children
        .map((e) => Application.fromXML(e.toString()))
        .toList();
    return AuthenticateRequest(
        xsi: xsi,
        xmlns: xmlns,
        airline: airline,
        eventToken: eventToken,
        applications: applicationsList,
        platformDefinedParameter: platformDefinedParameter,
        messageID: int.parse(messageID));
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="#$xsi" messageName="$messageName">
        <$messageName airline = "$airline" eventToken = "$eventToken" platformDefinedParameter = "$platformDefinedParameter">
            <applicationList>
                ${applications.map((e) => e.toXML()).toList().join("")}
            </applicationList>
        </$messageName>
    </cupps>'''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName";
  }
}

class Application {
  final String applicationName;
  final String applicationVersion;
  final String? applicationData;

  Application(
      {required this.applicationName,
      required this.applicationVersion,
      this.applicationData});

  factory Application.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String applicationName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "applicationName")
        .value;
    String applicationVersion = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "applicationName")
        .value;
    String applicationData = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "applicationData",
            orElse: () => XmlAttribute(XmlName.fromString(""), ""))
        .value;

    return Application(
        applicationName: applicationName,
        applicationVersion: applicationVersion,
        applicationData: applicationData);
  }

  String toXML() {
    if ((applicationData ?? "").isEmpty) {
      return '''<application applicationName="$applicationName" applicationVersion="$applicationVersion"/>''';
    } else {
      return '''<application applicationName="$applicationName" applicationVersion="$applicationVersion" applicationData="$applicationData"/>''';
    }
  }

  @override
  String toString() {
    return "$applicationName : $applicationVersion : $applicationData";
  }
}
