import 'package:xml/xml.dart';

import '../classes.dart';

class InterfaceLevelsAvailableResponse extends CuppsBase {
  final String result;
  final List<InterfaceLevel> interfaceLevels;

  InterfaceLevelsAvailableResponse({
    String? xmlns,
    String? xsi,
    required this.result,
    required this.interfaceLevels,
  }) : super(messageID: 1, messageName: "interfaceLevelsAvailableRequest");

  factory InterfaceLevelsAvailableResponse.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    document.findElements("messageName");

    String result = document.rootElement.firstElementChild!.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;
    List<InterfaceLevel> interfaceLevels = document
        .rootElement.childElements.first.childElements
        .map((e) => InterfaceLevel.fromXML(e.toString()))
        .toList();

    return InterfaceLevelsAvailableResponse(
      interfaceLevels: interfaceLevels,
      result: result,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns:xsi="$xsi" messageName="$messageName" messageID="$messageID">
        <$messageName result="$result">
            ${interfaceLevels.map((e) => e.toXML()).join("")}
        </$messageName>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName : $xsi ${interfaceLevels.length}";
  }
}

class InterfaceLevel {
  final String level;
  final String wsLocalPath;
  final String xsdVersion;

  InterfaceLevel({
    required this.level,
    required this.wsLocalPath,
    required this.xsdVersion,
  });

  factory InterfaceLevel.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String level = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "level")
        .value;
    String wsLocalPath = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "wsLocalPath")
        .value;
    String xsdVersion = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "xsdVersion")
        .value;

    return InterfaceLevel(
        level: level, wsLocalPath: wsLocalPath, xsdVersion: xsdVersion);
  }

  String toXML() {
    return '''
    <interfaceLevel level="$level" wsLocalPath="$wsLocalPath" xsdVersion="$xsdVersion" />
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$level : $wsLocalPath : $xsdVersion";
  }
}
