import 'package:xml/xml.dart';
import '../classes.dart';

class PrintResponse extends CuppsBase {
  final String result;
  final List<PrintDocumentResult> printDocumentResults;

  PrintResponse({
    required int messageID,
    required String messageName,
    required this.result,
    required this.printDocumentResults,
  }) : super(messageID: messageID, messageName: messageName);

  factory PrintResponse.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    String result = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;
    List<XmlElement> printDocsElements =
        document.rootElement.findAllElements("printDocumentResult").toList();
    List<PrintDocumentResult> printDocsResults = printDocsElements
        .map((e) => PrintDocumentResult.fromXML(e.toXmlString()))
        .toList();

    return PrintResponse(
        messageName: messageName,
        messageID: int.parse(messageID),
        result: result,
        printDocumentResults: printDocsResults);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
    <printResponse result="$result">
      ${printDocumentResults.map((e) => e.toXML()).join()}
    </printResponse>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName: $result";
  }
}

class PrintDocumentResult {
  final String result;
  final String documentID;

  PrintDocumentResult({
    required this.result,
    required this.documentID,
  });

  factory PrintDocumentResult.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String result = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;
    String documentID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "documentID")
        .value;
    return PrintDocumentResult(result: result, documentID: documentID);
  }

  String toXML() {
    return '''<printDocumentResult documentID="$documentID" result="$result"/>''';
  }
}
