import 'package:xml/xml.dart';
import 'Devices/bc_device.dart';
import 'Devices/be_device.dart';
import 'Devices/bg_device.dart';
import 'Devices/bp_device.dart';
import 'Devices/bt_device.dart';
import 'Devices/dd_device.dart';
import 'Devices/ms_device.dart';
import 'Devices/oc_device.dart';
import 'Devices/pr_device.dart';
import 'Devices/zi_device.dart';
import 'Devices/zl_device.dart';

class CuppsBase {
  final String xmlns = "http://www.cupps.aero/cupps/01.03";
  final int messageID;
  final String messageName;
  final String xsi = "http://www.w3.org/2001/XMLSchemainstance";

  CuppsBase({required this.messageID, required this.messageName});

  // CuppsBase({this.xmlns ="http://www.cupps.aero/cupps/01.03", required this.messageID, required this.messageName,this.xsi = "http://www.w3.org/2001/XMLSchema−instance"});

  factory CuppsBase.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    // String xmlns = document.rootElement.attributes.firstWhere((p0) => p0.name.toString().startsWith("xmlns")).value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    // String xsi = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns:xsi").value;

    return CuppsBase(
      messageName: messageName,
      messageID: int.parse(messageID),
      // xsi: xsi,
      // xmlns: xmlns,
    );
  }

  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
    </cupps>
    ''';
  }
}

class PlatformDevices {
  final List<BcDevice> bcDevices;
  final List<BeDevice> beDevices;
  final List<BgDevice> bgDevices;
  final List<BpDevice> bpDevices;
  final List<BtDevice> btDevices;
  final List<DdDevice> ddDevices;
  final List<MsDevice> msDevices;
  final List<OcDevice> ocDevices;
  final List<PrDevice> prDevices;
  final List<ZiDevice> ziDevices;
  final List<ZlDevice> zlDevices;

  PlatformDevices({
    required this.bcDevices,
    required this.beDevices,
    required this.bgDevices,
    required this.bpDevices,
    required this.btDevices,
    required this.ddDevices,
    required this.msDevices,
    required this.ocDevices,
    required this.prDevices,
    required this.ziDevices,
    required this.zlDevices,
  });

  factory PlatformDevices.empty() {
    return PlatformDevices(
        bcDevices: [],
        beDevices: [],
        bgDevices: [],
        bpDevices: [],
        btDevices: [],
        ddDevices: [],
        msDevices: [],
        ocDevices: [],
        prDevices: [],
        ziDevices: [],
        zlDevices: []);
  }

  void removeDevice(String deviceIndex) {
    bcDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    beDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    bgDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    bpDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    btDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    ddDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    msDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    ocDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    prDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    ziDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
    zlDevices.removeWhere((element) => element.deviceIndex == deviceIndex);
  }

  bool get hasBC => bcDevices.isNotEmpty;
  bool get hasBG => bgDevices.isNotEmpty;
  bool get hasBP => bpDevices.isNotEmpty;
  bool get hasBT => btDevices.isNotEmpty;
  bool get hasMS => msDevices.isNotEmpty;
  bool get hasOC => ocDevices.isNotEmpty;
  bool get hasPR => prDevices.isNotEmpty;
  bool get hasBE => beDevices.isNotEmpty;
  bool get hasDD => ddDevices.isNotEmpty;
  bool get hasZL => zlDevices.isNotEmpty;
  bool get hasZI => ziDevices.isNotEmpty;

  bool get hasDevice => hasBC ||hasBG ||hasBP ||hasBT ||hasMS ||hasOC ||hasPR ||hasBE ||hasDD ||hasZI ||hasZL;

  List<DeviceType> get availableDeviceTypes {
    List<DeviceType> list = [];
    if(hasBP) list.add(DeviceType.bp);
    if(hasBT) list.add(DeviceType.bt);
    if(hasBG) list.add(DeviceType.bg);
    if(hasBC) list.add(DeviceType.bc);
    if(hasMS) list.add(DeviceType.ms);
    if(hasOC) list.add(DeviceType.oc);
    if(hasPR) list.add(DeviceType.pr);
    if(hasDD) list.add(DeviceType.dd);
    if(hasZL) list.add(DeviceType.zl);
    if(hasZI) list.add(DeviceType.zi);
    if(hasBE) list.add(DeviceType.be);
    return list;
  }



}



XmlAttribute orElseEmpty() => XmlAttribute(XmlName(""), "");

enum DeviceType { bp, bt, bc, be, oc, ms, pr, bg, dd, zl, zi }

extension Details on DeviceType {
  String get parameter {
    switch (this) {
      case DeviceType.bp:
        return "bpDeviceParameter";
      case DeviceType.bt:
        return "btDeviceParameter";
      case DeviceType.bc:
        return "bcDeviceParameter";
      case DeviceType.be:
        return "beDeviceParameter";
      case DeviceType.oc:
        return "ocDeviceParameter";
      case DeviceType.ms:
        return "msDeviceParameter";
      case DeviceType.pr:
        return "prDeviceParameter";
      case DeviceType.bg:
        return "bgDeviceParameter";
      case DeviceType.dd:
        return "ddDeviceParameter";
      case DeviceType.zl:
        return "zlDeviceParameter";
      case DeviceType.zi:
        return "ziDeviceParameter";
      default:
        return "ocDeviceParameter";
    }
  }

  String get name {
    switch (this) {
      case DeviceType.bp:
        return "bp";
      case DeviceType.bt:
        return "bt";
      case DeviceType.bc:
        return "bc";
      case DeviceType.be:
        return "be";
      case DeviceType.oc:
        return "oc";
      case DeviceType.ms:
        return "ms";
      case DeviceType.pr:
        return "pr";
      case DeviceType.bg:
        return "bg";
      case DeviceType.dd:
        return "dd";
      case DeviceType.zl:
        return "zl";
      case DeviceType.zi:
        return "zi";
      default:
        return "";
    }
  }
}

class CuppsCommandResponse {
  final String msg;
  final bool status;
  final String? rawXML;
  CuppsCommandResponse({required this.status, required this.msg, this.rawXML});

  @override
  String toString() {
    return "${status ? "Operation Success" : "Operation Failed"} : $msg";
  }
}

class CuppsCommandException implements Exception {
  final String message;
  CuppsCommandException(this.message);

  @override
  String toString() {
    return message;
  }
}
