import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:artemis_cupps/classes/Notifications/messsage_id_error_event.dart';
import 'package:artemis_cupps/classes/Requests/device_query_request.dart';
import 'package:dart_ipify/dart_ipify.dart';
import 'package:get/get.dart';
import 'package:network_info_plus/network_info_plus.dart';

import '../../classes/classes.dart';


import 'package:xml/xml.dart';

import 'logger.dart';
const  minPlatformMsgID = 4294901760;
class SocketService {
  SocketService();

  String logIdentifier = "Unkown";
  String logSuffix = "Unkown";
  int messageID = 0;
  Socket? _socket;
  bool retry = true;
  void Function(CuppsBase, String)? requestHandler;
  void Function()? onConnectionClose;
  void Function()? onConnectionLost;
  String temp = "";
  final Map<int, Completer<String>> _requests = {};

  void enableRetry(){
    retry = true;
  }
  void disableRetry(){
    retry = false;
  }

  bool get isConnected => (_socket != null);

  Future <bool?> initialize(
      {required String ip,
      required int port,
      required void Function(CuppsBase, String) rh,
      required void Function() onClose,
      required void Function() onLost,
      required String logID,
      required String logSuff,
      }) async {
    if (_socket != null) {
      await _socket!.close();
    }
    messageID = 1;
    requestHandler = rh;
    onConnectionClose = onClose;
    logIdentifier = logID;
    logSuffix = logSuff;
    onConnectionLost = onLost;
    try {
      late RawSocketOption option;
      if(Platform.isAndroid) {
        option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0x1, /* SO_KEEPALIVE */ 0x0009, true);
      }else{
        // option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0xffff, /* SO_KEEPALIVE */ 0x0008, true);
        option = RawSocketOption.fromInt(/* SOL_SOCKET */ 0xffff, /* SO_KEEPALIVE */ 0x0008, 5);
      }
      Socket s = await Socket.connect(ip, port, timeout: const Duration(seconds: 10)).catchError((e){
        throw Exception("$e");
      }).timeout(const Duration (seconds: 12),onTimeout: (){
        throw Exception("TimeOut");
      }).then((s){
        s.listen(_onSocketMessage, cancelOnError: true, onError: (e) {
          log("ERROR ON SOCKET $e");
          disconnect();
        },onDone: (){
          log("Socket DONE");
          disconnect();
          // if(shouldTryReconnect){
          //   log("Reconnecting");
          //   onLost();
          //   // initialize(ip: ip, port: port, rh: rh, onClose: onClose, onLost: onLost, logID: logID, logSuff: logSuff);
          // }
        });
        _socket = s;
        return s;
      });
      _socket = s;
      return true;
    } catch (e) {
      // Get.snackbar("Connection Error", e.toString());
      log("ERROR ON LISTENING TO SOCKET $e");
      return false;

    }
    log("$logIdentifier Socket Service Connected");
  }

  Future<String> sendSocketMessage(String data,{bool shouldLog = true}) async {
    try {
      if (isConnected && _socket!=null) {
        final completer = Completer<String>();
        final requestId = messageID;
        messageID=(messageID%minPlatformMsgID)+1;
        _requests[requestId] = completer;
        XmlDocument formattedDOC = XmlDocument.parse(data);

        String formattedXML = formattedDOC.toXmlString(pretty: true,preserveWhitespace: (_)=>true);
        String header = generateHeaderForXML(formattedXML);
        String request = header + formattedXML;
        // print("Sending $messageID --->$data");
        if(shouldLog) {
          await Logger.writeToLog(header + formattedXML, from: "Application", to: logIdentifier, logSuffix: logSuffix);
        }
        _socket!.write(request);

        return completer.future;
      } else {
        String xmlRes ='''<?xml version="1.0"?><!--Message sending time 2021-11-18 00:19:29.698-->
<cupps xmlns="http://www.cupps.aero/cupps/01.03" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" messageID="6" messageName="deviceReleaseResponse">
   <response result="Socket Closed"></response>
</cupps>''';
        return xmlRes;
      }
    } catch (e) {
      // log(data);
      log("Sending to Socket ERROR: $e");
      rethrow;
    }
  }

  Future<void> respond(String data) async {
    try {
      if (isConnected) {
        String header = generateHeaderForXML(data);
        String request = header + data;
        _socket!.write(request);
      } else {
        log("Socket Closed");
      }
    } catch (e) {
      // log(data);
      log("Sending to Socket ERROR: $e");
      rethrow;
    }
  }

  Future<void> _onSocketMessage(data) async {
    String xml = utf8.decode(data);
    String headerShouldBe = (temp + xml).split("<").first;
    bool messageENDED = xml.endsWith("</cupps>");
    String received = (temp + xml).replaceFirst(headerShouldBe, "");
    // print("received $received from Platform");

    try {
      if (messageENDED) {
        // print("MESSAGE COMPLETE");
        temp = "";
        String beautifulXMl = XmlDocument.parse(received).toXmlString(pretty: true,preserveWhitespace: (_)=>true);
        await Logger.writeToLog(beautifulXMl, to: "Application", from: logIdentifier, logSuffix: logSuffix);

        CuppsBase response = CuppsBase.fromXML(received);
        int messageID = response.messageID;
        if (_requests.containsKey(messageID)) {
          _requests[messageID]!.complete(received);
          _requests.remove(messageID);
        } else if(messageID>=minPlatformMsgID) {
          requestHandler!(response, received);
        }else{
          String msgIDExeptionXML= MessageIDErrorEvent(messageID: messageID, messageName: 'messageIDErrorEvent').toXML();
          sendSocketMessage(msgIDExeptionXML);
          disconnect();
        }
      } else {
        log("MESSAGE IS COMING");
        temp += xml;
      }
    } catch (e) {
      log("$e--->$received");
    }
  }

  static String generateHeaderForXML(String xml) {
    int length = utf8.encode(xml).length;
    String temp = length.toRadixString(16).padLeft(6, "0");
    String header = "0100" + temp;
    return header;
  }

  Future<void> disconnect({bool disableRetry = false}) async {
    try {
      if (_socket != null) {
        // _instance.socket!.close();
        // await Future.delayed(const Duration(milliseconds: 100));
        if(disableRetry) {
          retry = false;
        }
        await _socket!.close();
        log("Socket Service Closed");
        // _socket?.destroy();
        log("Socket Service Destroyed");
        _socket = null;
        onConnectionClose!();
      }
    }catch (e){
      onConnectionClose!();
      log("Disconnect Exception $e");
    }
  }

  static Future<String?> getLocalIp() {
    return NetworkInfo().getWifiIP();
  }
  static Future<String?> getPublicIp() {
    return Ipify.ipv4();
  }

  void setConnectionChecker({Duration timeout = const Duration(seconds: 10),Duration interval = const Duration(seconds: 30),
    required Function retryFunction}) {
    if(retry) {
      try {
        Future.delayed(interval, () {
          if(logIdentifier=="Platform") {
            log("$logIdentifier Connection Checker ${DateTime.now()}");
            DeviceQueryRequest req = DeviceQueryRequest(messageID: messageID);
            if(isConnected){
              sendSocketMessage(req.toXML(), shouldLog: true).timeout(timeout, onTimeout: () {
                log("$logIdentifier Connection Checker Timeout **Reconnect** ${DateTime.now()}");
                retryFunction();
                return "";
              }).then((value) {
                if (value.isNotEmpty && !value.contains("Socket Closed")) {
                  log("$logIdentifier Connection Checker Connection OK ${DateTime.now()}");
                  setConnectionChecker(timeout:timeout,interval: interval,retryFunction: retryFunction);
                }
              });
            }else{
              retryFunction();
            }
          }else{
            log("$logIdentifier Connection Checker ${DateTime.now()}");
            DeviceStatusRequest req = DeviceStatusRequest(messageID: messageID);
            if(isConnected) {
              sendSocketMessage(req.toXML(), shouldLog: false).timeout(timeout, onTimeout: () {
                log("$logIdentifier Connection Checker Timeout **Reconnect** ${DateTime.now()}");
                retryFunction();
                return "";
              }).then((value) {
                if (value.isNotEmpty && !value.contains("Socket Closed")) {
                  log("$logIdentifier Connection Checker Connection OK ${DateTime.now()}");
                  setConnectionChecker(timeout: timeout, interval: interval, retryFunction: retryFunction);
                }
              });
            }else{
              retryFunction();
            }
          }

        });
      } catch (e) {
        log("$logIdentifier Connection Checker Error **Reconnect** ${DateTime.now()}");
        onConnectionLost!();
      }
    }

  }
}
